import numpy as np
import xarray as xr

from numpy.lib.stride_tricks import sliding_window_view
from attrs import define, field, validators, Factory
from typing import Dict

import functools
import logging

# import utils
import mvcm.conf as conf
import dev.conf_xr as conf_xr
import mvcm.scene as scn
import mvcm.preprocess_thresholds as preproc
import mvcm.restoral as restoral

import importlib

_RTD = 180./np.pi
_DTR = np.pi/180
_scene_list = ['Ocean_Day', 'Ocean_Night', 'Land_Day', 'Land_Night', 'Day_Snow', 'Night_Snow', 'Coast_Day',
               'Land_Day_Desert', 'Antarctic_Day', 'Polar_Day_Snow', 'Polar_Day_Desert', 'Polar_Day_Ocean',
               'Polar_Day_Desert_Coast', 'Polar_Day_Coast', 'Polar_Day_Land', 'Polar_Night_Snow',
               'Polar_Night_Land', 'Polar_Night_Ocean', 'Land_Day_Desert_Coast', 'Land_Day_Coast', ]
#               # 'Desert', 'Australia']

# this is used for testing, eventually we want to remove it
importlib.reload(preproc)
importlib.reload(conf)
importlib.reload(restoral)

logger = logging.getLogger('__name__')


def restoral_flag(bits):
    qa_bit = bits['qa_bit']
    test_bit = bits['test_bit']
    return np.logical_and(qa_bit, np.logical_not(test_bit))


@define(kw_only=True, slots=True)
class CloudTests(object):
    """
    """
    data: xr.Dataset = field(validator=[validators.instance_of(xr.Dataset), ])
    scene_name: str = field(validator=[validators.instance_of(str),
                                       validators.in_(_scene_list)])
    thresholds: str = field(validator=[validators.instance_of(Dict), ])
    scene_idx: tuple = field(init=False,
                             default=Factory(lambda self: tuple(np.nonzero(self.data[self.scene_name] == 1)),
                                             takes_self=True),
                             validator=[validators.instance_of(tuple), ])
    pixels_in_scene: bool = field(init=False,
                                  default=Factory(lambda self: self.scene_idx[0].shape[0] != 0,
                                                  takes_self=True),
                                  validator=[validators.instance_of(bool), ])

    def run_if_test_exists_for_scene(test_name):
        def decorate(func):

            @functools.wraps(func)
            def wrapper(self, *args, **kwargs):
                if test_name not in self.thresholds[self.scene_name]:
                    test_bit = np.zeros(args[-1].shape)
                    return args[-1], test_bit
                else:
                    kwargs['confidence'] = np.ones(self.data.latitude.shape)
                    kwargs['qa_bit'] = np.zeros(self.data.latitude.shape)
                    kwargs['test_bit'] = np.zeros(self.data.latitude.shape)
                    kwargs['thresholds'] = self.thresholds[self.scene_name][test_name]
                    return func(self, *args, **kwargs)

            return wrapper
        return decorate

    def compute_test_bits(self,
                          rad: np.ndarray,
                          thr: np.ndarray,
                          ineq: str) -> np.ndarray:

        rad = rad.ravel()
        if ineq == '<=':
            idx = np.nonzero((rad <= thr) & (self.data[self.scene_name].values[self.scene_idx] == 1))
        if ineq == '<':
            idx = np.nonzero((rad < thr) & (self.data[self.scene_name].values[self.scene_idx] == 1))
        if ineq == '>':
            idx = np.nonzero((rad > thr) & (self.data[self.scene_name].values[self.scene_idx] == 1))
        if ineq == '>=':
            idx = np.nonzero((rad >= thr) & (self.data[self.scene_name].values[self.scene_idx] == 1))

        test_bits = np.zeros(rad.shape)
        test_bits[idx] = 1

        return test_bits

    @run_if_test_exists_for_scene('11um_Test')
    def test_11um(self,
                  band: str,
                  cmin: np.ndarray,
                  **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]

            # idx = np.nonzero((self.data[band].values >= threshold['thr'][1]) &
            #                  (self.data[self.scene_name] == 1))
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, threshold['thr'][1], '>=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, threshold['thr'])

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    # THIS FUNCTION DOES NOT HAVE THE TEST_BIT
    @run_if_test_exists_for_scene('Surface_Temperature_Test')
    def surface_temperature_test(self,
                                 band: str,
                                 viirs_data: xr.Dataset,
                                 cmin: np.ndarray,
                                 **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')

            rad = self.data[band].values[self.scene_idx]
            sfcdif = viirs_data.geos_sfct.values[self.scene_idx] - rad
            thr = preproc.thresholds_surface_temperature(viirs_data, threshold, self.scene_idx)

            # idx = np.nonzero((sfcdif < thr[1, :]) & (self.data[self.scene_name] == 1))
            # kwargs['test_bit'][idx] = 1
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(sfcdif, thr[:, 1], '<')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(sfcdif, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('SST_Test')
    def sst_test(self,
                 band31: str,
                 band32: str,
                 cmin: np.ndarray,
                 **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            m31 = self.data[band31].values - 273.16
            bt_diff = self.data[band31].values - self.data[band32].values
            sst = self.data.sst.values - 273.16
            cosvza = np.cos(self.data.sensor_zenith.values*_DTR)

            c = threshold['coeffs']

            modsst = 273.16 + c[0] + c[1]*m31 + c[2]*bt_diff*sst + c[3]*bt_diff*((1.0/cosvza) - 1.0)
            sfcdif = self.data.sst.values - modsst

            # idx = np.nonzero((sfcdif < threshold['thr'][1]) & (self.data[self.scene_name] == 1))
            # kwargs['test_bit'][idx] = 1
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(sfcdif[self.scene_idx],
                                                                        threshold['thr'][1], '<')
            print(f'Testing "{self.scene_name}"\n')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(sfcdif[self.scene_idx], threshold['thr'])

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('8.6-11um_Test')
    def bt_diff_86_11um(self,
                        band: str,
                        cmin: np.ndarray,
                        **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((self.data[band].values < threshold['thr'][1]) &
            #                  (self.data[self.scene_name] == 1))
            # kwargs['test_bit'][idx] = 1
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, threshold['thr'][1], '<')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, threshold['thr'])

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('11-12um_Cirrus_Test')
    def test_11_12um_diff(self,
                          band: str,
                          cmin: np.ndarray,
                          **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            thr = preproc.thresholds_11_12um(self.data, threshold, self.scene_name, self.scene_idx)
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((rad <= thr[1, :]) & (self.data[self.scene_name] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])
        kwargs['test_bit'] = kwargs['test_bit'].reshape(self.data[band].shape)

        return cmin, restoral_flag(kwargs)

    # THIS NEEDS TO BE TESTED
    @run_if_test_exists_for_scene('11-4um_BT_Difference_Test_Ocean')
    def bt_difference_11_4um_test_ocean(self,
                                        band: str,
                                        cmin: np.ndarray,
                                        **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            thr = preproc.bt_diff_11_4um_thresholds(self.data, threshold, self.scene_idx)
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((rad >= thr[1, :]) & (self.data[self.scene_name] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_dble(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    # THIS NEEDS TO BE TESTED
    @run_if_test_exists_for_scene
    def bt_difference_11_4um_test_land(self,
                                       band1: str,
                                       band2: str,
                                       cmin: np.ndarray,
                                       test_name: str = '11-4um_BT_Difference_Test_Land') -> np.ndarray:

        confidence = np.ones(self.data.M15.shape)
        qa_bit = np.zeros(self.data[band1].shape)
        test_bit = np.zeros(self.data[band1].shape)
        threshold = self.thresholds[self.scene_name][test_name]

        scene_flag = scn.find_scene(self.data, self.thresholds['Sun_Glint']['bounds'][3])

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            qa_bit[self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            if self.scene_name == 'Land_Night':
                australia = self.data.Australia.values
                ndvi = self.data.ndvi.values
                coast = scene_flag['coast']
                sh_lake = scene_flag['sh_lake']
                idx = np.where(((australia == 0) & (ndvi < threshold['ndvi']) & (coast == 0)) |
                               ((australia == 1) & (ndvi < threshold['ndvi_australia']) & (coast == 0)))
                not_idx = np.ones(australia.shape, dtype=bool)
                not_idx[idx] = False

                rad11_12 = self.data[band1].values
                rad11_4 = self.data[band2].values
                thr = preproc.get_nl_thresholds(self.data, threshold, coast=False)
                idx_lake = np.where((sh_lake == 1) | (rad11_12 < 0))
                # test_bit[self.scene_idx] = 1  # THIS NEEDS TO BE CALCULATED PROPERLY
                thr[:3, idx_lake] = thr[:3, idx_lake] + 2
                temp_thr = np.squeeze(thr[:, idx[0]])
                temp_rad = rad11_4[idx]
                confidence[idx] = conf.conf_test_new(temp_rad, temp_thr)

                thr = preproc.land_night_thresholds(self.data, threshold, coast=True)
                temp_thr = np.squeeze(thr[:, not_idx])
                temp_rad = rad11_4[not_idx]
                thr[0:3, idx_lake[0], idx_lake[1]] = thr[0:3, idx_lake[0], idx_lake[1]] - 1
                thr[3:6, idx_lake[0], idx_lake[1]] = thr[3:6, idx_lake[0], idx_lake[1]] + 1

                confidence[not_idx] = conf.conf_test_dble(rad11_4[not_idx], thr[:, not_idx])

            if self.scene_name in ['Polar_Night_Land', 'Polar_Day_Snow', 'Day_Snow']:
                rad11_12 = self.data[band1].values[self.scene_idx]
                rad11_4 = self.data[band2].values[self.scene_idx]
                thr = preproc.polar_night_thresholds(self.data, threshold, self.scene_name,
                                                     test_name, self.scene_idx)
                confidence[self.scene_idx] = conf.conf_test_new(rad11_4, thr)

            if self.scene_name in ['Polar_Night_Snow']:
                rad11_12 = self.data[band1].values
                rad11_4 = self.data[band2].values
                idx_top = np.nonzero(self.data.M15.values > threshold['bt_thr'])
                thr_top = preproc.polar_night_thresholds(self.data, threshold['top'], self.scene_name,
                                                         test_name, idx_top)
                idx_bottom = np.nonzero((self.data.M15.values > threshold['bt_thr']) &
                                        (self.data.geos_tpw > threshold['tpw_thr']))
                thr_bottom = preproc.polar_night_thresholds(self.data, threshold['bottom'], self.scene_name,
                                                            test_name, idx_top)

                confidence[idx_top] = conf.conf_test_new(rad11_4[idx_top], thr_top)
                confidence[idx_bottom] = conf.conf_test_new(rad11_4[idx_bottom], thr_bottom)

            if self.scene_name in ['Night_Snow']:
                rad11_12 = self.data[band1].values
                rad11_4 = self.data[band2].values
                idx_top = np.nonzero(self.data.M15.values > threshold['bt_thr'])
                thr_top = threshold['thr']
                idx_bottom = np.nonzero((self.data.M15.values > threshold['bt_thr']) &
                                        (self.data.geos_tpw > threshold['tpw_thr']))
                thr_bottom = preproc.polar_night_thresholds(self.data, threshold['bottom'], self.scene_name,
                                                            test_name, idx_top)

                confidence[idx_top] = conf.conf_test_new(rad11_4[idx_top], thr_top)
                confidence[idx_bottom] = conf.conf_test_new(rad11_4[idx_bottom], thr_bottom)

            cmin = np.fmin(cmin, confidence)

        return cmin, test_bit

    # This test requires 7.3um band that is not available for VIIRS
    @run_if_test_exists_for_scene
    def midlevel_cloud_test():
        pass

    # This test requires 7.3um band that is not available for VIIRS
    @run_if_test_exists_for_scene
    def water_vapor_cloud_test():
        pass

    @run_if_test_exists_for_scene('11um_Variability_Test')
    def variability_11um_test(self,
                              band: str,
                              cmin: np.ndarray,
                              **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']
        var = np.zeros([self.data.M15.shape[0], self.data.M15.shape[1], 9])
        radshape = self.data.M15.shape

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values

            test = (sliding_window_view(np.pad(rad, [1, 1], mode='constant'), (3, 3)) -
                    np.expand_dims(rad, (2, 3)))
            var[np.abs(test).reshape(radshape[0], radshape[1], 9) < threshold['variability']] = 1
            var = var.sum(axis=2)

            idx = np.where(var == 9)
            kwargs['test_bit'][idx] = 1
            thr = np.array(threshold['thr'])
            kwargs['confidence'][idx] = conf.conf_test_new(rad[idx], thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    # THIS FUNCTION NEEDS THE TEST_BIT DOUBLE CHECKED
    @run_if_test_exists_for_scene('11-4um_Oceanic_Stratus_Test')
    def oceanic_stratus_11_4um_test(self,
                                    band: str,
                                    cmin: np.ndarray,
                                    **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            print(f'Testing "{self.scene_name}"\n')
            # rad = self.data[band].values[self.scene_idx]
            rad = self.data[band].values
            thr = threshold['thr']
            if self.scene_name in ['Land_Day_Desert', 'Land_Day_Desert_Coast', 'Polar_Day_Desert',
                                   'Polar_Day_Desert_Coast']:
                kwargs['confidence'][self.scene_idx] = conf.conf_test_dble(rad[self.scene_idx], thr)

            # these scenes have not been implemented yet... and it might not be necessary
            # elif self.scene_name in ['Land_Night', 'Polar_Night_Land', 'Polar_Day_Snow', 'Polar_Night_Snow',
            #                          'Day_Snow', 'Night_Snow', 'Antarctic_Day']:
            #     pass
            else:
                scene_flag = scn.find_scene(self.data, self.thresholds['Sun_Glint']['bounds'][3])
                idx = np.nonzero((self.data[band].values >= threshold['thr'][1]) &
                                 (self.data[self.scene_name] == 1) &
                                 (scene_flag['sunglint'] == 0))
                kwargs['test_bit'][idx] = 1
                idx = tuple(np.nonzero((self.data[self.scene_name] == 1) &
                                       (scene_flag['sunglint'] == 0)))
                kwargs['confidence'][idx] = conf.conf_test_new(rad[idx], thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('NIR_Reflectance_Test')
    def nir_reflectance_test(self,
                             band: str,
                             cmin: np.ndarray,
                             **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            thr = preproc.thresholds_NIR(self.data, self.thresholds, self.scene_name,
                                         'NIR_Reflectance_Test', self.scene_idx)
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((rad <= thr[1, :]) &
            #                  (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('Vis/NIR_Ratio_Test')
    def vis_nir_ratio_test(self,
                           band: str,
                           cmin: np.ndarray,
                           **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            # I'm not using the self.scene_idx here because it messes up with the indexing of the
            # confidence array and I don't want to think of a solution at the moment. I will need
            # to figure out the logic to make it work once I'm at the stage where I want to optimize
            # the code
            kwargs['qa_bit'][self.scene_idx] = 1
            rad = self.data[band].values  # [self.scene_idx]
            solar_zenith = self.data.solar_zenith.values  # [self.scene_idx]
            sunglint = scn.find_scene(self.data, self.thresholds['Sun_Glint']['bounds'][3])['sunglint']

            idx = np.nonzero((solar_zenith <= 85) & (sunglint == 1))

            thr_no_sunglint = np.array([threshold['thr'][i] for i in range(8)])
            thr_sunglint = np.array([self.thresholds['Sun_Glint']['snglnt'][i] for i in range(8)])
            print(thr_no_sunglint)
            print(thr_sunglint)
            # tmp = self.thresholds['Sun_Glint']['snglnt']
            # thr_sunglint = np.array([tmp[0], tmp[1], tmp[2], tmp[3], tmp[4], tmp[5], 1])
            kwargs['confidence'][sunglint == 0] = conf.conf_test_dble(rad[sunglint == 0], thr_no_sunglint)
            kwargs['confidence'][idx] = conf.conf_test_dble(rad[idx], thr_sunglint)

            idx = np.nonzero(((rad < thr_no_sunglint[1]) | (rad > thr_no_sunglint[4])) &
                             (self.data[self.scene_name].values == 1) & (sunglint == 0))
            kwargs['test_bit'][tuple(idx)] = 1

            idx = np.nonzero(((rad < thr_sunglint[1]) | (rad > thr_sunglint[4])) &
                             ((self.data[self.scene_name].values == 1) &
                              (solar_zenith <= 85) & (sunglint == 1)))
            kwargs['test_bit'][tuple(idx)] = 1

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('1.6_2.1um_NIR_Reflectance_Test')
    def test_16_21um_reflectance(self,
                                 band: str,
                                 cmin: np.ndarray,
                                 **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            thr = preproc.thresholds_NIR(self.data, self.thresholds, self.scene_name,
                                         '1.6_2.1um_NIR_Reflectance_Test', self.scene_idx)
            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((rad <= thr[1, :]) &
            #                  (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    # this test needs some TLC
    @run_if_test_exists_for_scene('Visible_Reflectance_Test')
    def visible_reflectance_test(self,
                                 band: str,
                                 cmin: np.ndarray,
                                 **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']
        logger.warning('The pytest now seems to be working now but it used to fail for this function. ' +
                       'If it fails again, go check the test before turning this function inside out')

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            print(f'Testing "{self.scene_name}"\n')
            thr, rad = preproc.vis_refl_thresholds(self.data, self.thresholds, self.scene_name,
                                                   self.scene_idx)

            # idx = np.nonzero((rad <= thr[1, :]) & (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_name]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('GEMI_Test')
    def gemi_test(self,
                  band: str,
                  cmin: np.ndarray,
                  **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            thr = preproc.gemi_thresholds(self.data, threshold, self.scene_name,
                                          self.scene_idx)
            rad = self.data[band].values[self.scene_idx]

            # idx = np.nonzero((rad > thr[1, :]) & (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_name]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '>')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('1.38um_High_Cloud_Test')
    def test_1_38um_high_clouds(self,
                                band: str,
                                cmin: np.ndarray,
                                **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            kwargs['qa_bit'][self.scene_idx] = 1
            if self.scene_name in ['Ocean_Day', 'Polar_Day_Ocean']:
                thr = preproc.thresholds_1_38um_test(self.data, self.thresholds, self.scene_name,
                                                     self.scene_idx)
            else:
                # return cmin, kwargs['test_bit']
                thr = np.full((len(self.scene_idx[0]), 4), threshold['thr'][:4]).T

            print(f'Testing "{self.scene_name}"\n')
            rad = self.data[band].values[self.scene_idx]
            # idx = np.nonzero((rad <= thr[1, :]) &
            #                  (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)

    @run_if_test_exists_for_scene('4-12um_BTD_Thin_Cirrus_Test')
    def thin_cirrus_4_12um_BTD_test(self,
                                    band: str,
                                    cmin: np.ndarray,
                                    **kwargs) -> np.ndarray:

        threshold = kwargs['thresholds']

        if (threshold['perform'] is True and self.pixels_in_scene is True):
            thr = preproc.polar_night_thresholds(self.data, self.thresholds, self.scene_name,
                                                 '4-12um_BTD_Thin_Cirrus_Test', self.scene_idx)
            rad = self.data[band].values[self.scene_idx]

            # idx = np.nonzero((rad <= thr[1, :]) &
            #                  (self.data[self.scene_name].values[self.scene_idx] == 1))
            # tmp_bit = kwargs['test_bit'][self.scene_idx]
            # tmp_bit[idx] = 1
            # kwargs['test_bit'][self.scene_idx] = tmp_bit
            kwargs['test_bit'][self.scene_idx] = self.compute_test_bits(rad, thr[1, :], '<=')
            kwargs['confidence'][self.scene_idx] = conf.conf_test_new(rad, thr)

        cmin = np.fmin(cmin, kwargs['confidence'])

        return cmin, restoral_flag(kwargs)


class ComputeTests(CloudTests):

    def __init__(self,
                 data: xr.Dataset,
                 scene_name: str,
                 thresholds: Dict) -> None:
        super().__init__(data, scene_name, thresholds)

    def run_tests(self):
        cmin_G1 = np.ones(self.data.M01.shape)
        cmin_G2 = np.ones(self.data.M01.shape)
        cmin_G3 = np.ones(self.data.M01.shape)
        cmin_G4 = np.ones(self.data.M01.shape)
        cmin_G5 = np.ones(self.data.M01.shape)

        # Group 1
        cmin_G1 = self.test_11um('M15', cmin_G1, test_name='11um_Test')
        cmin_G1 = self.sst_test('M15', 'M16', cmin_G1, test_name='SST_Test')

        # Group 2
        cmin_G2 = self.bt_diff_86_11um('M14-M15', cmin_G2, test_name='8.6-11um_Test')
        cmin_G2 = self.test_11_12um_diff('M15-M16', cmin_G2, test_name='11-12um_Cirrus_Test')
        cmin_G2 = self.oceanic_stratus_11_4um_test('M15-M13', cmin_G2,
                                                   test_name='11-4um_Oceanic_Stratus_Test')

        # Group 3
        cmin_G3 = self.nir_reflectance_test('M07', cmin_G3, test_name='NIR_Reflectance_Test')
        cmin_G3 = self.vis_nir_ratio_test('M07-M05ratio', cmin_G3, test_name='Vis/NIR_Ratio_Test')
        cmin_G3 = self.nir_reflectance_test('M10', cmin_G3, test_name='1.6_2.1um_NIR_Reflectance_Test')
        cmin_G3 = self.visible_reflectance_test('M128', cmin_G3, test_name='Visible_Reflectance_Test')
        cmin_G3 = self.gemi_test('GEMI', cmin_G3, test_name='GEMI_Test')

        # Group 4
        cmin_G4 = self.test_1_38um_high_clouds('M09', cmin_G4, test_name='1.38um_High_Cloud_Test')

        # Group 5
        cmin_G5 = self.thin_cirrus_4_12um_BTD_test('M13-M16', cmin_G5,
                                                   test_name='4-12um_BTD_Thin_Cirrus_Test')

        cmin = cmin_G1 * cmin_G2 * cmin_G3 * cmin_G4 * cmin_G5

        return cmin

    def clear_sky_restoral(self,
                           cmin: np.ndarray) -> np.ndarray:

        total_bit = np.full(self.data.M01.shape, 3)
        sunglint_angle = self.thresholds['Sun_Glint']['bounds'][3]
        scene_flags = scn.find_scene(self.data, sunglint_angle)

        cmin_tmp = cmin + 0
        idx = np.nonzero((scene_flags['water'] == 1) & (scene_flags['ice'] == 0) &
                         (scene_flags['uniform'] == 1) & (cmin <= 0.99) & (cmin >= 0.05))
        cmin[idx] = restoral.spatial(self.data, self.thresholds['Sun_Glint'], scene_flags, cmin_tmp)[idx]

        cmin_tmp = cmin + 0
        idx = np.nonzero((scene_flags['water'] == 1) & (scene_flags['sunglint'] == 1) &
                         (scene_flags['uniform'] == 1) & (cmin <= 0.95))
        cmin[idx] = restoral.sunglint(self.data, self.thresholds['Sun_Glint'], total_bit, cmin_tmp)[idx]

        """
        idx = np.nonzero((scene_flags['day'] == 1) & (scene_flags['land'] == 1) &
                         (scene_flags['snow'] == 0) & (scene_flags['ice'] == 0) &
                         (cmin <= 0.95))
        cmin[idx] = restoral.land(self.data, self.thresholds, scene_flags, cmin)[idx]

        idx = np.nonzero((scene_flags['day'] == 1) & (scene_flags['land'] == 1) &
                         (scene_flags['coast'] == 1) & (scene_flags['snow'] == 0) &
                         (scene_flags['ice'] == 0))
        cmin[idx] = restoral.coast(self.data, self.thresholds, scene_flags, cmin)[idx]
        """

        return cmin












